package de.hendriklipka.q3master;

import java.util.Properties;
import java.io.*;

import org.log4j.*;

public class Configuration
{
    private Category CAT=Category.getInstance(Configuration.class.getName());
    private Category LOG_CAT=Category.getInstance("q3master.config");

    private static Configuration _instance=new Configuration();

    private String _log4jConfigFile="cfg/q3master.lcf";
    private String _listenAddress=null;
    private int _listenPort=27950;
    private int _listenTimeout=5000;
    private int _serverTimeout=11*60*1000;
    private int _serverQueryTimeout=2000;
    private int _serverQueryTries=3;
    private boolean _filterProtocol=true;
    private boolean _filterFull=true;
    private boolean _filterEmpty=true;

    public String getListenAddress()
    {
        return _listenAddress;
    }

    public int getListenPort()
    {
        return _listenPort;
    }

    public int getListenTimeout()
    {
        return _listenTimeout;
    }

    public int getServerTimeout()
    {
        return _serverTimeout;
    }

    public int getServerQueryTimeout()
    {
        return _serverQueryTimeout;
    }

    public int getServerQueryTries()
    {
        return _serverQueryTries;
    }

    public String getLog4jConfigFile()
    {
        return _log4jConfigFile;
    }

    public boolean filterProtocol()
    {
        return _filterProtocol;
    }

    public boolean filterFull()
    {
        return _filterFull;
    }

    public boolean filterEmpty()
    {
        return _filterEmpty;
    }

    public static Configuration getInstance()
    {
        return _instance;
    }

    public void readConfigFile(String fName)
    {
        try
        {
            Properties props=new Properties();
            props.load(new FileInputStream(fName));
            if (props.containsKey("server.timeout"))
            {
                _serverTimeout=getIntValue(props,"server.timeout")*1000*60;
            }
            if (props.containsKey("serverquery.timeout"))
            {
                _serverQueryTimeout=getIntValue(props,"serverquery.timeout")*1000;
            }
            if (props.containsKey("serverquery.tries"))
            {
                _serverQueryTries=getIntValue(props,"serverquery.tries");
            }
            if (props.containsKey("listen.port"))
            {
                _listenPort=getIntValue(props,"listen.port");
            }
            if (props.containsKey("listen.address"))
            {
                _listenAddress=props.getProperty("listen.address");
            }
            if (props.containsKey("listen.timeout"))
            {
                _listenTimeout=getIntValue(props,"listen.timeout");
            }
            if (props.containsKey("filter.protocol"))
            {
                _filterProtocol=getBooleanValue(props,"filter.protocol");
            }
            if (props.containsKey("filter.full"))
            {
                _filterFull=getBooleanValue(props,"filter.full");
            }
            if (props.containsKey("filter.empty"))
            {
                _filterEmpty=getBooleanValue(props,"filter.empty");
            }
            if (props.containsKey("log.log4j.configfile"))
            {
                _log4jConfigFile=props.getProperty("log.log4j.configfile");
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }

    public void dumpConfig()
    {
        CAT.debug("server.timeout="+_serverTimeout);
        CAT.debug("serverquery.timeout="+_serverQueryTimeout);
        CAT.debug("serverquery.tries="+_serverQueryTries);
        CAT.debug("listen.port="+_listenPort);
        CAT.debug("listen.address="+_listenAddress);
        CAT.debug("listen.timeout="+_listenTimeout);
        CAT.debug("filter.protocol="+_filterProtocol);
        CAT.debug("filter.full="+_filterFull);
        CAT.debug("filter.empty="+_filterEmpty);
        CAT.debug("log.log4j.configfile="+_log4jConfigFile);
    }

    protected boolean getBooleanValue(Properties props, String name)
    {
        String s=props.getProperty(name);
        if (null==s)
            return false;
        s=s.toLowerCase();
        if (s.equals("yes")||s.equals("1")||s.equals("true"))
            return true;
        return false;
    }

    protected int getIntValue(Properties props, String name)
    {
        try
        {
            String s=props.getProperty(name);
            s=s.trim();
            return Integer.parseInt(s);
        }
        catch (Exception ex)
        {
            LOG_CAT.error(ex);
        }
        return -1;
    }
}